==> snippet_library-1.2.6.tgz: package/app/components/questions/match/match.component.js <==
import matchTemplate from './match.template.html'

/* @ngInject */
let matchComponent = {
  template: matchTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  controller: function ($rootScope, $scope, $http, $timeout, $state, $localStorage, $stateParams, $log, xxtea, exerciseService, appConstants, popupService, loaderService) {
    // Local config
    $scope.config = {}

    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({ status: 'false', data: 'Bad token.' })
      } else if (response.status === 404) {
        $log.error({ status: 'false', data: 'Bad request.' })
      } else if (response.status === 405) {
        $log.error({ status: 'false', data: 'Method not allowed.' })
      } else {
        $log.error({ status: 'false', data: response.statusText })
      }

    };

    // Popup
    $scope.popupVisible = false
    let popupVisibilityChange = $rootScope.$on('popup.changed', function (elem, data) {
      $scope.popupVisible = data.active
    })

    $scope.step = 0 // Current question (step)
    // User info
    $scope.info = {
      time_attempted: new Date().getTime(), // Time when user started the exercise
      time_finished: undefined // Time when the user finished the exercise
    }
    // Questions related data
    $scope.current = {
      set_a_index: 0, // Current index in questions array
      set_b_index: 0, // Current index in answers array
      set_b_copy: [], // Copy of the answers array
      matches: [], // User matches answers
      question: { // for app start we should assign the set_a/set_b empty array
        set_a: [], // Questions array
        set_b: [] // Answers array
      },
      correct: undefined, // Is user answer correct
      answers: [], // User attempt answers
      correctAnswers: 0, // Overall correct answers
      attempts: 0, // Attempt number
      tryAgain: false, // Is in try again state
      nextQuestion: false // Is in next question state
    }

    /**
         * Reset state of the exercise to beginning
         * @param {boolean} tryAgain Determines if the action should be try again or next question
         * @returns {void}
         */
    $scope.resetExercise = (tryAgain) => {
      popupService.hide()
      $scope.current.set_a_index = 0
      $scope.current.set_b_index = 0
      $scope.current.matches = []
      $scope.current.tryAgain = false
      $scope.current.nextQuestion = false
      $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
      if (!tryAgain) {
        $scope.current.question.attempts = 0
      }
    }

    /**
         * Setting up the exercise
         * @param {object} exerciseData exercise object
         * @returns {void}
         */
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.current.correct = $scope.exercise.interaction.correct //xxtea.decryptFromBase64($scope.exercise.interaction.correct, appConstants.tea); // Decrypt xxTea and parse JSON
      $scope.current.answers = $scope.exercise.interaction.questions.map((item) => { return { question: item.question_id, attempts: [] } }) // Map answers to correct array
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
      stepWatch()
    }

    // Watching function, watches for step changes
    function stepWatch() {
      popupService.hide()
      if ($scope.exercise) {
        if ($scope.exercise.interaction.questions.length <= $scope.step) {
          $scope.finishExercise()
        } else {
          $scope.resetExercise(false)
          $scope.current.question = $scope.exercise.interaction.questions[$scope.step]
          $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
          $localStorage.currentActiveQuestion = $scope.step + 1
        }
      }
    }
    $scope.$watch('step', (n, o) => { stepWatch() }) // Watching for current answer ( step )

    /**
         * Switch to next question
         * @returns {void}
         */
    $scope.nextQuestion = () => { $scope.step++ }
    $scope.nextSubQuestion = () => { $scope.current.set_a_index++ }

    /**
         * @param {object} val The input value
         * @returns {bool} Return true if the input value is Integer otherwise false
         */
    $scope.isNumber = (val) =>{
      return parseInt(val)
    }

    /**
         * Checks if current question is already matched
         * @returns {boolean} true/false
         */
    $scope.isCurrentMatched = () => {
      let match = $scope.current.matches.find((m) => {
        return m.a.tile_data === $scope.current.question.set_a[$scope.current.set_a_index].tile_data
      })
      return match
    }

    /**
         * Checks if question with index is already matched
         * @param {int} index question index
         * @param {string} setType question type (a,b)
         * @returns {boolean} true/false
         */
    $scope.isQuestionMatched = (index, setType) => {
      let match = undefined
      if (setType === 'a') {
        match = $scope.current.matches.find((m) => {
          return m.a.tile_data === $scope.current.question.set_a[index].tile_data
        })
      } else if (setType === 'b') {
        match = $scope.current.matches.find((m) => {
          return m.b.tile_data === $scope.current.question.set_b[index].tile_data
        })
      }
      return match
    }

    /**
         * Checks if all questions are matched
         * @returns {boolean} true/false
         */
    $scope.isAllMatched = () => {
      return $scope.current.set_b_copy.length === 0 && !$scope.current.tryAgain && !$scope.current.nextQuestion
    }

    /**
         * Gets questions that are not matched
         * @returns {array<object>} List of Sets
         */
    $scope.getNotMatchedQuestions = () => {
      let list = $scope.current.question.set_b.filter((b1) => {
        var filter = $scope.current.matches.find((b2) => b1.tile_data === b2.b.tile_data)
        return angular.isUndefined(filter)
      })
      return list
    }

    /**
         * Select current question
         * @returns {void}
         */
    $scope.matchQuestion = () => {
      let setA = $scope.current.question.set_a[$scope.current.set_a_index]
      let setB = $scope.getNotMatchedQuestions()[$scope.current.set_b_index]
      $scope.current.matches.push({ a: setA, b: setB, index: angular.copy($scope.current.set_a_index), index_b: angular.copy($scope.current.set_b_index)})
      $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
    }

    /**
         * Unselect current question
         * @returns {void}
         */
    $scope.unmatchQuestion = () => {
      let indexToRemove = $scope.current.matches.findIndex((m) => {
        return $scope.current.question.set_a[$scope.current.set_a_index].tile_data === m.a.tile_data
      })

      $scope.current.matches.splice(indexToRemove, 1)
      $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
    }

    /**
         * Check if the current question is last
         * @returns {boolean} true if the question is last
         */
    $scope.isLastQuestion = () => {
      return $scope.current.question.set_a.length - 1 === $scope.current.set_a_index
    }

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @param {string} last defines if it is the last question and everything is matched
         * @returns {object} string or function
         */
    $scope.currentAction = (type, last) => {
      if ($scope.current.question.answered === true) {
        if (type !== 'text' && $scope.current.question.answered_popup !== true) {
          $scope.finishExercise()
        }
        else if ($scope.current.question.answered_popup) {
          popupService.show('You have already answered this question!', 'semi-smile', false)
        }
        return type === 'text' ? 'Next' : $state.go('result')
      }
      else {
        if (type && last) {
          if ($scope.isAllMatched()) {
            if ($scope.isCurrentMatched()) {
              return type === 'text' ? 'Unselect as match' : $scope.unmatchQuestion()
            } else {
              return type === 'text' ? 'Select as match' : $scope.matchQuestion()
            }
          }
        }
        if ($scope.isAllMatched()) {
          return type === 'text' ? 'Check all answers' : $scope.checkAllAnswers()
        } else if ($scope.current.tryAgain && !$scope.current.nextQuestion) {
          return type === 'text' ? 'Try again' : $scope.resetExercise(true)
        } else if ($scope.current.nextQuestion) {
          return type === 'text' ? 'Next' : $state.go('result')
        } else {
          if ($scope.isCurrentMatched()) {
            return type === 'text' ? 'Unselect as match' : $scope.unmatchQuestion()
          } else {
            return type === 'text' ? 'Select as match' : $scope.matchQuestion()
          }
        }
      }
    }


    /**
         * @param {string} type text or function
         * @returns {void}
         */

    $scope.nextAction = (type) => {
      function submit() {
        if (!$scope.current.nextQuestion) {
          $scope.checkAllAnswers()
        }
        $scope.nextQuestion()
      }
      if ($scope.isLastQuestion()) {
        return type === 'text' ? 'Submit' : submit()
      } else {
        return type === 'text' ? 'Next' : $scope.nextSubQuestion()
      }
    }

    /**
         * Checks all answers and show the feedback in popup directive
         * @returns {void}
         */
    $scope.checkAllAnswers = () => {
      $scope.current.attempts++
      $scope.valid = $scope.current.matches.filter((m, i) => {
        return m.b.tile_data === xxtea.decryptFromBase64($scope.current.correct[$scope.step].answers[m.index], appConstants.tea)
      })
      $scope.total = $scope.current.question.set_a.length
      $scope.isCorrect = $scope.valid.length === $scope.total
      $scope.userAnswer = $scope.current.matches.map((m) => {
        var ans = {}
        ans[m.a.tile_data] = m.b.tile_data
        return ans
      })
      $scope.attempt = $scope.current.answers.find((attemptAnswer) => { return attemptAnswer.question === $scope.current.question.question_id }) // Finds array with all user attempts for current question
      $scope.attempt.attempts.push({ answer: $scope.userAnswer, correct: $scope.isCorrect, attempt_number: $scope.attempt.attempts.length + 1 })// Adds the attempt to an attempts array

      $scope.current.correctAnswers = $scope.valid.length
      if ($scope.current.question.answered === true) {
        popupService.show('You have already answered this question!', 'semi-smile', false)
      }
      else {
        if ($scope.isCorrect) {
          $scope.current.question.answered = true
          popupService.show('Very good!', 'happy-smile', true) // Show popup
          $scope.current.nextQuestion = true
        } else { // Answer is not correct
          if ($scope.current.attempts === 1) {
            popupService.show('Some of your answers are incorrect.', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
          } else if ($scope.current.attempts === 2) {
            popupService.show('Some of your answers are incorrect.', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
          } else if ($scope.current.attempts === 3) {
            let html = '<br/>' + $scope.current.question.set_b.map((b, i) => { return (i + 1) + '. ' + b.tile_data }).join('<br/>')
            popupService.show('The correct answers are: ' + html, 'sad-smile', false) // Show popup
            $scope.current.tryAgain = false
            $scope.current.nextQuestion = true
            $scope.current.question.answered = true
          }
        }
      }
    }

    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      $scope.info.time_finished = new Date().getTime() // Get time when user finished exercise
      $scope.correctAnswersCount = $scope.current.correctAnswers // Correct answers counter

      //const score = Math.round(((correctAnswersCount / $scope.exercise.interaction.questions.length) * 100)); // Get user score in percent
      $scope.overallQuestionsCounts = 0
      angular.forEach($scope.exercise.interaction.questions, function (q) { $scope.overallQuestionsCounts += q.set_a.length })
      $scope.score = Math.round((($scope.correctAnswersCount / $scope.overallQuestionsCounts) * 100))
      const data = {
        'score': $scope.score,
        'user': 24275,
        'answers': $scope.current.answers,
        'time_attempted': $scope.info.time_attempted,
        'time_finished': $scope.info.time_finished,
        'exercise_id': $scope.exercise.exercise_id,
        'playlist_id': $scope.exercise.playlist.playlist_id,
        'context': $scope.exercise.context.context,
        'context_id': $scope.exercise.context.context_id,
        'id': $scope.exercise.exercise_id,
        'group_id': $scope.exercise.group_id
      }

      // $http.post(appConstants.api + '/exerciseStoreScore', data).then(() => {
      //   let nextExercise = undefined
      //   if ($localStorage.playlist) {
      //     if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
      //       nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
      //     }
      //   }
      //
      //   exerciseService.setVal({
      //     'score': score,
      //     'totalQuestions': overallQuestionsCounts,
      //     'correctAnswer': correctAnswersCount,let
      //     'nextExercise': nextExercise
      //   })
      //
      //   $state.go('result') // Go to result page
      // }, $scope.errorCallback)

      $scope.nextExercise = undefined
      exerciseService.setVal({
        'score': $scope.score,
        'totalQuestions': $scope.overallQuestionsCounts,
        'correctAnswer': $scope.correctAnswersCount,
        'nextExercise': $scope.nextExercise
      })
      $scope.current.question.answered_popup = true
    }

    // Get exercise data and setup
    $scope.$watch('[$ctrl.questionsId,$ctrl.sid]', (n, o) => {
      if ($stateParams.id) {
        $http({ method: 'GET', url: appConstants.api + '/exercise/' + $scope.$ctrl.questionsId + '/' + $scope.$ctrl.sid })
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'cloze', id: $stateParams.id, sid: $stateParams.sid }

            // $timeout(() => { $state.go('exercise') })
          }, errorCallback)
      } else { // If we have loaded playlist then load the exercise from playlist
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default matchComponent

==> snippet_library-2.0.7.tgz: package/app/components/questions/match/match.component.js <==
import matchTemplate from './match.template.html'

/* @ngInject */
let matchComponent = {
  template: matchTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  controller: function ($rootScope, $scope, $compile, $element, $http, $timeout, $location, $state, $localStorage, $stateParams, $log, xxtea, exerciseService, appConstants, popupService, loaderService) {
    // Local config
    $scope.config = {}

    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({ status: 'false', data: 'Bad token.' })
      } else if (response.status === 404) {
        $log.error({ status: 'false', data: 'Bad request.' })
      } else if (response.status === 405) {
        $log.error({ status: 'false', data: 'Method not allowed.' })
      } else {
        $log.error({ status: 'false', data: response.statusText })
      }

    };

    // Popup
    $scope.popupVisible = false
    let popupVisibilityChange = $rootScope.$on('popup.changed', function (elem, data) {
      $scope.popupVisible = data.active
    })

    $scope.step = 0 // Current question (step)
    // User info
    $scope.info = {
      time_attempted: new Date().getTime(), // Time when user started the exercise
      time_finished: undefined // Time when the user finished the exercise
    }
    // Questions related data
    $scope.current = {
      set_a_index: 0, // Current index in questions array
      set_b_index: 0, // Current index in answers array
      set_b_copy: [], // Copy of the answers array
      matches: [], // User matches answers
      question: { // for app start we should assign the set_a/set_b empty array
        set_a: [], // Questions array
        set_b: [] // Answers array
      },
      correct: undefined, // Is user answer correct
      answers: [], // User attempt answers
      correctAnswers: 0, // Overall correct answers
      attempts: 0, // Attempt number
      tryAgain: false, // Is in try again state
      nextQuestion: false // Is in next question state
    }

    /**
         * Reset state of the exercise to beginning
         * @param {boolean} tryAgain Determines if the action should be try again or next question
         * @returns {void}
         */
    $scope.resetExercise = (tryAgain) => {
      popupService.hide()
      $scope.current.set_a_index = 0
      $scope.current.set_b_index = 0
      $scope.current.matches = []
      $scope.current.tryAgain = false
      $scope.current.nextQuestion = false
      $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
      if (!tryAgain) {
        $scope.current.question.attempts = 0
      }
    }

    /**
         * Setting up the exercise
         * @param {object} exerciseData exercise object
         * @returns {void}
         */
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.current.correct = $scope.exercise.interaction.correct //xxtea.decryptFromBase64($scope.exercise.interaction.correct, appConstants.tea); // Decrypt xxTea and parse JSON
      $scope.current.answers = $scope.exercise.interaction.questions.map((item) => { return { question: item.question_id, attempts: [] } }) // Map answers to correct array
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
      stepWatch()
    }

    // Watching function, watches for step changes
    function stepWatch() {
      popupService.hide()
      if ($scope.exercise) {
        if ($scope.exercise.interaction.questions.length <= $scope.step) {
          $scope.finishExercise()
        } else {
          $scope.resetExercise(false)
          $scope.current.question = $scope.exercise.interaction.questions[$scope.step]
          $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
          $localStorage.currentActiveQuestion = $scope.step + 1
        }
      }
    }
    $scope.$watch('step', (n, o) => { stepWatch() }) // Watching for current answer ( step )

    /**
         * Switch to next question
         * @returns {void}
         */
    $scope.nextQuestion = () => { $scope.step++ }
    $scope.nextSubQuestion = () => { $scope.current.set_a_index++ }

    /**
         * @param {object} val The input value
         * @returns {bool} Return true if the input value is Integer otherwise false
         */
    $scope.isNumber = (val) =>{
      return parseInt(val)
    }

    /**
         * Checks if current question is already matched
         * @returns {boolean} true/false
         */
    $scope.isCurrentMatched = () => {
      let match = $scope.current.matches.find((m) => {
        return m.a.tile_data === $scope.current.question.set_a[$scope.current.set_a_index].tile_data
      })
      return match
    }

    /**
         * Checks if question with index is already matched
         * @param {int} index question index
         * @param {string} setType question type (a,b)
         * @returns {boolean} true/false
         */
    $scope.isQuestionMatched = (index, setType) => {
      let match = undefined
      if (setType === 'a') {
        match = $scope.current.matches.find((m) => {
          return m.a.tile_data === $scope.current.question.set_a[index].tile_data
        })
      } else if (setType === 'b') {
        match = $scope.current.matches.find((m) => {
          return m.b.tile_data === $scope.current.question.set_b[index].tile_data
        })
      }
      return match
    }

    /**
         * Checks if all questions are matched
         * @returns {boolean} true/false
         */
    $scope.isAllMatched = () => {
      return $scope.current.set_b_copy.length === 0 && !$scope.current.tryAgain && !$scope.current.nextQuestion
    }

    /**
         * Gets questions that are not matched
         * @returns {array<object>} List of Sets
         */
    $scope.getNotMatchedQuestions = () => {
      let list = $scope.current.question.set_b.filter((b1) => {
        var filter = $scope.current.matches.find((b2) => b1.tile_data === b2.b.tile_data)
        return angular.isUndefined(filter)
      })
      return list
    }

    /**
         * Select current question
         * @returns {void}
         */
    $scope.matchQuestion = () => {
      let setA = $scope.current.question.set_a[$scope.current.set_a_index]
      let setB = $scope.getNotMatchedQuestions()[$scope.current.set_b_index]
      $scope.current.matches.push({ a: setA, b: setB, index: angular.copy($scope.current.set_a_index), index_b: angular.copy($scope.current.set_b_index)})
      $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
    }

    /**
         * Unselect current question
         * @returns {void}
         */
    $scope.unmatchQuestion = () => {
      let indexToRemove = $scope.current.matches.findIndex((m) => {
        return $scope.current.question.set_a[$scope.current.set_a_index].tile_data === m.a.tile_data
      })

      $scope.current.matches.splice(indexToRemove, 1)
      $scope.current.set_b_copy = $scope.getNotMatchedQuestions()
    }

    /**
         * Check if the current question is last
         * @returns {boolean} true if the question is last
         */
    $scope.isLastQuestion = () => {
      return $scope.current.question.set_a.length - 1 === $scope.current.set_a_index
    }

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @param {string} last defines if it is the last question and everything is matched
         * @returns {object} string or function
         */
    $scope.currentAction = (type, last) => {
      if ($scope.current.question.answered === true) {
        if (type !== 'text' && $scope.current.question.answered_popup !== true) {
          $scope.finishExercise()
        }
        else if ($scope.current.question.answered_popup) {
          popupService.show('You have already answered this question!', 'semi-smile', false)
        }
        if ($scope.loc !== 'practice') {
          return type === 'text' ? 'Next' : $state.go('result')
        } else {
          $element.replaceWith($compile('<result></result>')($scope))
        }
      }
      else {
        if (type && last) {
          if ($scope.isAllMatched()) {
            if ($scope.isCurrentMatched()) {
              return type === 'text' ? 'Unselect as match' : $scope.unmatchQuestion()
            } else {
              return type === 'text' ? 'Select as match' : $scope.matchQuestion()
            }
          }
        }
        if ($scope.isAllMatched()) {
          return type === 'text' ? 'Check all answers' : $scope.checkAllAnswers()
        } else if ($scope.current.tryAgain && !$scope.current.nextQuestion) {
          return type === 'text' ? 'Try again' : $scope.resetExercise(true)
        } else if ($scope.current.nextQuestion) {
          if ($scope.loc !== 'practice') {
            return type === 'text' ? 'Next' : $state.go('result')
          } else {
            $element.replaceWith($compile('<result></result>')($scope))
          }
        } else {
          if ($scope.isCurrentMatched()) {
            return type === 'text' ? 'Unselect as match' : $scope.unmatchQuestion()
          } else {
            return type === 'text' ? 'Select as match' : $scope.matchQuestion()
          }
        }
      }
    }


    /**
         * @param {string} type text or function
         * @returns {void}
         */

    $scope.nextAction = (type) => {
      function submit() {
        if (!$scope.current.nextQuestion) {
          $scope.checkAllAnswers()
        }
        $scope.nextQuestion()
      }
      if ($scope.isLastQuestion()) {
        return type === 'text' ? 'Submit' : submit()
      } else {
        return type === 'text' ? 'Next' : $scope.nextSubQuestion()
      }
    }

    /**
         * Checks all answers and show the feedback in popup directive
         * @returns {void}
         */
    $scope.checkAllAnswers = () => {
      $scope.current.attempts++
      $scope.valid = $scope.current.matches.filter((m, i) => {
        return m.b.tile_data === xxtea.decryptFromBase64($scope.current.correct[$scope.step].answers[m.index], appConstants.tea)
      })
      $scope.total = $scope.current.question.set_a.length
      $scope.isCorrect = $scope.valid.length === $scope.total
      $scope.userAnswer = $scope.current.matches.map((m) => {
        var ans = {}
        ans[m.a.tile_data] = m.b.tile_data
        return ans
      })
      $scope.attempt = $scope.current.answers.find((attemptAnswer) => { return attemptAnswer.question === $scope.current.question.question_id }) // Finds array with all user attempts for current question
      $scope.attempt.attempts.push({ answer: $scope.userAnswer, correct: $scope.isCorrect, attempt_number: $scope.attempt.attempts.length + 1 })// Adds the attempt to an attempts array

      $scope.current.correctAnswers = $scope.valid.length
      if ($scope.current.question.answered === true) {
        popupService.show('You have already answered this question!', 'semi-smile', false)
      }
      else {
        if ($scope.isCorrect) {
          $scope.current.question.answered = true
          popupService.show('Very good!', 'happy-smile', true) // Show popup
          $scope.current.nextQuestion = true
        } else { // Answer is not correct
          if ($scope.current.attempts === 1) {
            popupService.show('Some of your answers are incorrect.', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
          } else if ($scope.current.attempts === 2) {
            popupService.show('Some of your answers are incorrect.', 'semi-smile', false) // Show popup
            $scope.current.tryAgain = true
            $scope.current.nextQuestion = false
          } else if ($scope.current.attempts === 3) {
            let html = '<br/>' + $scope.current.correct[0].answers.map((b, i) => { return (i + 1) + '. ' + xxtea.decryptFromBase64(b, appConstants.tea) }).join('<br/>')
            if ($scope.isCurrentMatched().b.tile_type === 'image') {
              html = '<br/>' + $scope.current.correct[0].answers.map((b, i) => { return (i + 1) + '. ' + '<img class="center-block popup-img" src=' + xxtea.decryptFromBase64(b, appConstants.tea) + '>'}).join('<br/>')
            }
            popupService.show('The correct answers are: ' + html, 'sad-smile', false) // Show popup
            $scope.current.tryAgain = false
            $scope.current.nextQuestion = true
            $scope.current.question.answered = true
          }
        }
      }
    }

    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      $scope.info.time_finished = new Date().getTime() // Get time when user finished exercise
      $scope.correctAnswersCount = $scope.current.correctAnswers // Correct answers counter

      //const score = Math.round(((correctAnswersCount / $scope.exercise.interaction.questions.length) * 100)); // Get user score in percent
      $scope.overallQuestionsCounts = 0
      angular.forEach($scope.exercise.interaction.questions, function (q) { $scope.overallQuestionsCounts += q.set_a.length })
      $scope.score = Math.round((($scope.correctAnswersCount / $scope.overallQuestionsCounts) * 100))
      const data = {
        'score': $scope.score,
        'answers': $scope.current.answers,
        'time_attempted': $scope.info.time_attempted,
        'time_finished': $scope.info.time_finished,
        'exercise_id': $scope.exercise.exercise_id,
        'playlist_id': $scope.exercise.playlist.playlist_id,
        'context': $scope.exercise.context.context,
        'context_id': $scope.exercise.context.context_id,
        'id': $scope.exercise.exercise_id,
        'group_id': $scope.exercise.group_id
      }

      $http.post(appConstants.api + '/exercise/score/store', data, appConstants.http_config).then(() => {
      }, $scope.errorCallback)

      $scope.nextExercise = undefined
      if ($localStorage.playlist) {
        if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
          $scope.nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
        }
      }
      exerciseService.setVal({
        'score': $scope.score,
        'totalQuestions': $scope.overallQuestionsCounts,
        'correctAnswer': $scope.correctAnswersCount,
        'nextExercise': $scope.nextExercise
      })
      $scope.current.question.answered_popup = true
    }

    // Get exercise data and setup
    // Get exercise data and setup
    $scope.$watch('[$ctrl.questionsId,$ctrl.sid]', (n, o) => {
      $scope.loc = $location.absUrl().replace('/' + $scope.location, '').split('/').slice(-1).pop()
      if ($stateParams.id) {
        $http.get(appConstants.api + '/exercise/' + $scope.$ctrl.questionsId + '/' + $scope.$ctrl.sid, appConstants.http_config)
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'match', id: $scope.$ctrl.questionsId, sid: $scope.$ctrl.sid }
            if ($scope.loc !== 'practice') {
              $timeout(() => { $state.go('exercise') })
            }
          }, errorCallback)
      } else {
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default matchComponent
